# OE2500
###################################################################################################################################################
# Copyright 2023 Orchid Systems 
# -----------------------------
# Outline: script adds an item search button to the OE sales history screen, when selecting by Customer. 
#
###################################################################################################################################################
# Permitted for use with Orchid Systems Modules only.
# The code is supplied on an "as is" basis for training and demonstration purposes only and is not supported by Orchid Systems.
# If this code is deployed in a live production environment it is the responsibility of the End User to ensure 
# that it is operating as required.
###################################################################################################################################################

from accpac import *

def main(args):
	MyUI()

class myRec():
	def __init__(self):
		customer = ""
		item = ""
		yr = ""
		pd = 0

class MyUI(UI):

	def __init__(self):
		UI.__init__(self)
		self.currRec = myRec()
		self.thisRec = myRec()
		self.createScreen()

	def createScreen(self):
		self.getControlInfo('form|vlsOESHHD', self.onControlInfo)
		self.dsD = self.openDataSource("dsOESHHD")
		self.dsD.onAfter = self.dsD_onAfter
		
		self.dsARCUS = self.openDataSource("dsARCUS")

		self.show()

	def onControlInfo(self, info):
		f = info.find('FORM')

		g = info.find('vlsOESHHD')

		grid = self.getHostControl('vlsOESHHD')

		grid.setPosition(g.left, g.top + 400)

		self.grid = grid

		fld = self.addUIField('FINDITEM')
		fld.controlType = 'EDIT'
		fld.fieldType = 'STRING'
		fld.left = g.left
		fld.top = g.top - 50
		fld.width = 5000
		fld.labelWidth = 40
		fld.caption = 'Item Search:'
		fld.hasFinder = True
		fld.hasArrows = True
		fld.onClick = self.fldFindItem_onClick
		fld.onChange = self.fldFindItem_onChange
		fld.tabStop = True
		self.fldFindItem = fld

		## Add dummy invisible field.
		## Used so that the search field can be tabbed out of.
		fld = self.addUIField("NOTINUSE")
		fld.controlType = "EDIT"
		fld.fieldType = "STRING"
		fld.left = self.fldFindItem.left + self.fldFindItem.width + 500
		fld.top = self.fldFindItem.top
		fld.width = 0
		fld.labelWidth = 0
		fld.tabStop = True
		self.fldNotInUse = fld

		self.show()
		
	def dsD_onAfter(self, eventname, field, value):
		if eventname == "REFRESH":
			self.saveFilterValues()
			
	def fldFindItem_onChange(self, oldvalue, newvalue):
		fmtitem = self.getFormattedItem(newvalue)
		if not fmtitem:
			fmtitem = newvalue
			
		self.fldFindItem.setValue(fmtitem)
		self.goSearchFirst()

	def fldFindItem_onClick(self, btn):
		if btn == 0:
			self.goSearchFirst()
		elif btn == 1:
			self.goSearchPrev()
		elif btn == 2:
			self.goSearchNext()
		elif btn == 3:              
			self.goSearchLast()
		elif btn == 4:
			self.goSearchFinder()

	def getFilterString(self):
		unfmtitem = self.getUnformattedItem(self.fldFindItem.value.rstrip())

		sFilter = "CUSTOMER = \"{}\" AND ITEM = \"{}\"".format(self.customer, unfmtitem)
		sFilter += " AND ((YR > \"{}\") OR ((YR = \"{}\") AND (PERIOD>={})))".format(self.fromYear, self.fromYear, self.fromPeriod)
		sFilter += " AND ((YR < \"{}\") OR ((YR = \"{}\") AND (PERIOD<={})))".format(self.toYear, self.toYear, self.toPeriod)
		#showMessageBox(sFilter, "getFilterString")
		
		return sFilter
		
	def saveFilterValues(self):
		self.customer = self.dsARCUS.get("IDCUST").strip()
		self.getHostControlField("fecOESHHD_FromPeriod").getValue(self.getFromPeriod)
		self.getHostControlField("fecOESHHD_ToPeriod").getValue(self.getToPeriod)
		self.getHostControlField("fecCSFSC_ToYear").getValue(self.getToYear)
		self.getHostControlField("fecCSFSC_FromYear").getValue(self.getFromYear)
		
		self.setTimeout(10, self.afterGetFilterResults)

	def getCustomer(self, promise):
		## Promise has finished. Get the result obtained by the promise. Save it away.
		self.customer = str(promise.result)
		
	def getFromPeriod(self, promise):
		self.fromPeriod = str(promise.result)
		
	def getToPeriod(self, promise):
		self.toPeriod = str(promise.result)
		
	def getFromYear(self, promise):
		self.fromYear = str(promise.result)
		
	def getToYear(self, promise):
		self.toYear = str(promise.result)
		
	def afterGetFilterResults(self):
		t = "Customer [{}]\nFrom Year [{}]\nTo Year [{}]\nFrom Period [{}]\nTo Period [{}]".format(self.customer, self.fromYear, self.toYear, self.fromPeriod, self.toPeriod)
		#showMessageBox(t, "afterGetFilterResults")

	def goSearchFirst(self):
		sFilter = self.getFilterString()
		
		self.dsD.recordClear()
		self.dsD.browse(sFilter, 1)
		if self.dsD.fetch() == 0:
			pass
		else:
			message("Item [{}] not found in sales order details.".format(self.fldFindItem.value.rstrip()))
		
	def goSearchLast(self):
		self.dsD.put("CUSTOMER", self.customer)
		self.dsD.put("ITEM", "ZZZZZZZZZZZZZZZZZZZZZZZZ")
		self.dsD.put("YR", self.toYear)
		self.dsD.put("PERIOD", self.toPeriod)
		
		sFilter = self.getFilterString()
		self.dsD.browse(sFilter, 0)
		if self.dsD.fetch() == 0:
			pass
		else:
			message("Item [{}] not found in sales history details.".format(self.fldFindItem.value.rstrip()))
		
	def goSearchNext(self):
		self.getRecord()
		sFilter = self.getFilterString()
		self.dsD.browse(sFilter, 1) ## Filter forward.
		if self.dsD.fetch() == 0:
			if self.sameRecord():
				self.dsD.fetch()
		else:
			## try starting from the beginning.
			self.goSearchFirst()
		
	def goSearchPrev(self):
		self.getRecord()
		sFilter = self.getFilterString()
		self.dsD.browse(sFilter, 0)	## Filter in reverse.
		if self.dsD.fetch() == 0:
			if self.sameRecord():
				self.dsD.fetch()
		else:
			## try starting from the end.
			self.goSearchFirst()
		
	def goSearchFinder(self):
		finder = Finder()
		finder.viewID = "IC0310"
		finder.displayFields = "7,3,1"
		finder.returnFields = "7"	# Return the value field content.
		finder.onOK = self.onSearchFinder_ClickOK
		finder.onCancel = self.onSearchFinder_ClickCancel
		finder.show(self)

	def onSearchFinder_ClickOK(self, e):
		self.fldFindItem.setValue(e)
		self.goSearchFirst()
	
	def onSearchFinder_ClickCancel(self):
		ui_command("return", "1")
		
	def getFormattedItem(self, itemno):
		fmtitem = ""
		sSQL = "SELECT FMTITEMNO FROM ICITEM WHERE ITEMNO = '{0}' OR FMTITEMNO = '{0}'".format(itemno)
		v = openView("CS0120")
		v.recordClear()
		v.browse(sSQL, 1)
		if v.fetch() == 0:
			fmtitem = v.get("FMTITEMNO").strip()
		
		if v: v.close()

		return fmtitem
		
	def getUnformattedItem(self, itemno):
	
		unfmtitem = ""
		sSQL = "SELECT ITEMNO FROM ICITEM WHERE FMTITEMNO = '{}'".format(itemno)
		v = openView("CS0120")
		v.recordClear()
		v.browse(sSQL, 1)
		if v.fetch() == 0:
			unfmtitem = v.get("ITEMNO").strip()
		
		if v: v.close()

		return unfmtitem
		
	def getRecord(self):
		self.currRec.customer = self.dsD.get("CUSTOMER").strip()
		self.currRec.item = self.dsD.get("ITEM").strip()
		self.currRec.yr = self.dsD.get("YR").strip()
		self.currRec.pd = int(self.dsD.get("PERIOD"))
		
	def sameRecord(self):
		## compare records.
		self.thisRec.customer = self.dsD.get("CUSTOMER").strip()
		self.thisRec.item = self.dsD.get("ITEM").strip()
		self.thisRec.yr = self.dsD.get("YR").strip()
		self.thisRec.pd = int(self.dsD.get("PERIOD"))
		
		#t = "thisRec: [{}]\ncurrRec: [{}]".format(self.thisRec, self.currRec)
		#showMessageBox(t)
		
		if (self.thisRec.customer == self.currRec.customer) and \
			(self.thisRec.item == self.currRec.item) and \
			(self.thisRec.yr == self.currRec.yr) and \
			(self.thisRec.pd == self.currRec.pd):
			return True
			
		else:
			return False
			
